#include <cassert>
#include <fstream>
#include <iostream>

#include "Image.hxx"

Image::Image(int resX, int resY)
:
  m_resX(resX),
  m_resY(resY)
{
  assert (resX > 0 && resY > 0);
  std::cerr << "(Image): ResX = " << m_resX << ", resY = " << m_resY << std::endl;
  m_pixel = new Vec3f[m_resX*m_resY];
}

Image::~Image()
{
  delete [] m_pixel;
}

Image::Image()
{
}

Image::Image(const Image& o)
{
  operator=(o);
}

Image&
Image::operator=(const Image& o)
{
  return *this;
}

Vec3f&
Image::operator()(int x, int y)
{
  assert(x >= 0 && x < m_resX);
  assert(y >= 0 && y < m_resY);

  return m_pixel[y*m_resX+x];
}

void Image::WritePPM(const std::string& fileName)
{
  std::cerr << "(Image): Writing to file " << fileName << std::endl;
  std::ofstream file(fileName.c_str());

  if(!file.is_open())
    {
      std::cerr << "(Image): Could not open file " << fileName << std::endl;
      return;
    }

  file << "P3" << std::endl;
  file << m_resX << " " << m_resY << " " << 255 << std::endl;
  for (int y=m_resY-1;y>=0;y--)
    {
      for (int x=0;x<m_resX;x++)
	{
        if ((*this)(x,y)[0] < 0) (*this)(x,y)[0] = 0;
        if ((*this)(x,y)[1] < 0) (*this)(x,y)[1] = 0;
        if ((*this)(x,y)[2] < 0) (*this)(x,y)[2] = 0;
        if ((*this)(x,y)[0] > 255) (*this)(x,y)[0] = 255;
        if ((*this)(x,y)[1] > 255) (*this)(x,y)[1] = 255;
        if ((*this)(x,y)[2] > 255) (*this)(x,y)[2] = 255;
	  file
	    << (int)(255.99999999 * (*this)(x,y)[0]) << " "
	    << (int)(255.99999999 * (*this)(x,y)[1]) << " "
	    << (int)(255.99999999 * (*this)(x,y)[2]) << " "
	    << "\t";
	}
      file << std::endl;
      file << std::flush;
    }
  file.close();
 }
