#include "PhongShader.hxx"
#include "Primitive.hxx"

PhongShader::PhongShader(Scene* scene,
			 const Vec3f& am_c,
			 const Vec3f& di_c,
			 const Vec3f& sp_c,
			 float ka,
			 float kd,
			 float ks,
			 float ke)
  : Shader(scene),
    m_ambient_color(am_c),
    m_diffuse_color(di_c),
    m_specular_color(sp_c),
    m_ka(ka),
    m_kd(kd),
    m_ks(ks),
    m_ke(ke)
{
}

PhongShader::PhongShader()
  : Shader(0)
{
}

PhongShader::~PhongShader()
{
}

Vec3f
PhongShader::Shade(Ray& ray)
{
  // surface normal at hit point of object
  Vec3f normal = ray.hit()->GetNormal(ray);
  // turn to front if angle > 90°
  if(normal.dot(ray.direction()) < 0) {
    normal = normal * -1;
  }
  // direction of reflection
  Vec3f refl_dir = ray.direction() - normal * 2 * normal.dot(ray.direction());
  refl_dir.normalize();

  // ambient color term
  Vec3f amb_color = m_ambient_color * m_ka;

  Vec3f intensity, light_dir;

  // diffuse and specular color for each light source
  Vec3f diff_sum;
  Vec3f spec_sum;
  for(std::vector<Light*>::iterator it = m_scene->m_lights.begin();
    it != m_scene->m_lights.end(); it++) {
    Ray light_ray;
    (*it)->Illuminate(light_ray, intensity);
    diff_sum += intensity * light_ray.direction().dot(normal);
    Vec3f view_dir = ray.direction()*-1; // direction from hitpoint to viewer
    spec_sum += intensity * pow(fmax(view_dir.dot(refl_dir),0), m_ke);
  }
  Vec3f diff_color = m_diffuse_color * m_kd * diff_sum;
  Vec3f spec_color = m_specular_color * m_ks * spec_sum;

  return amb_color + diff_color + spec_color;
}
