#include "PointLight.hxx"

PointLight::PointLight(Scene* scene, const Vec3f& pos, const Vec3f& intensity)
  : Light(scene),
    m_pos(pos),
    m_intensity(intensity)
{
}

PointLight::~PointLight()
{
}

PointLight::PointLight()
  : Light(0),
    m_pos(Vec3f()),
    m_intensity(Vec3f())
{
}

bool
PointLight::Illuminate(Ray& ray, Vec3f& intensity)
{
  Vec3f dir = (ray.origin() + ray.direction() * (ray.t()-Epsilon)) - m_pos;
  float dist = dir.norm();
  dir.normalize();

  float c1 = 1, c2 = 0.5, c3 = 0;
  //float f_att = 1 / (dist*dist);
  float f_att = 1 / (c1 + c2*dist + c3*dist*dist);

  intensity = m_intensity * f_att;

  // store direction from light to hitpoint
  ray.setDir(dir);
  return true;
}

const Vec3f&
PointLight::position() const
{
  return m_pos;
}

const Vec3f&
PointLight::intensity() const
{
  return m_intensity;
}


